<?php
/*
Plugin Name: Find duplicates
Description: A plugin thas finds duplicate and similar posts based on their post_content-similarity. You can define the percentage of similarity, post type and post status. The plugin is a great utility to find duplicates that differ in only a few characters.
Version: 1.0
Author: Markus Seyer
Plugin URI: http://www.markusseyer.de
Author URI: http://www.markusseyer.de
*/

add_action('admin_menu', 'add_fd_page');
add_action('admin_init', 'fd_admin_init');

function add_fd_page()
{
	$page = add_management_page('Find duplicates', 'Find duplicates', 8, __FILE__, 'output_fd_page');
	add_action('admin_print_styles-' . $page, 'load_javascript');
}

function load_javascript()
{
	wp_enqueue_script('find-duplicates-js');
}


function fd_admin_init()
{
	wp_register_script('find-duplicates-js', plugins_url('/find-duplicates.js', __FILE__));
	load_plugin_textdomain('find-duplicates', false, basename(dirname(__FILE__)) . '/languages');
}

function output_fd_page()
{
	if (!current_user_can('edit_posts')) {
		wp_die(__('You do not have sufficient permissions to access this page.'));
	}
	include($plugin_dir_path = dirname(__FILE__) . '/find-duplicates-page.php');
}


function find_similar_posts($startnew, $search_parameter)
{

	ini_set("display_errors", true);
	set_time_limit(120);
	$log = "";
	$new_duplicates = array();
	$post2_limit = 2000;

	$options = get_option('find_duplicates_data', array());
	if (count($options) == 0 OR $startnew == 1) {
		$options = $search_parameter;
		$options['done'] = array();
		$options['post2_offset'] = 0;
		$options['found'] = array();
	}

	// get Posts
	$post_status = implode("','", $options['statuses']);
	$excludes = implode("','", $options['done']);
	$post2_offset = $options['post2_offset'];
	global $wpdb;
	$posts = $wpdb->get_results("SELECT ID,post_title,post_content FROM $wpdb->posts WHERE post_type='" . $options['types'] . "' AND post_status IN('" . $post_status . "') AND ID NOT IN('" . $excludes . "') LIMIT 1");
	$allPosts_count = $wpdb->get_results("SELECT COUNT(ID) as count FROM $wpdb->posts WHERE post_type='" . $options['types'] . "' AND post_status IN('" . $post_status . "') AND ID NOT IN('" . $excludes . "')");
	$allPosts_count = $allPosts_count[0]->count;
	$allPosts = $wpdb->get_results("SELECT ID,post_title,post_content FROM $wpdb->posts WHERE post_type='" . $options['types'] . "' AND post_status IN('" . $post_status . "') AND ID NOT IN('" . $excludes . "') LIMIT " . $post2_offset . "," . $post2_limit);
	if ($options['post2_offset'] <= $allPosts_count AND $post2_limit < $allPosts_count) {
		$options['post2_offset'] = $options['post2_offset'] + $post2_limit;
	} else {
		$options['post2_offset'] = 0;
	}

	// END get Posts

	foreach ($posts as $post) {
		$log .= "Comparing " . $post->ID . " (" . $post2_offset . "," . $post2_limit . ")<br />";
		foreach ($allPosts as $post2) {
			if ($post2->ID != $post->ID AND !in_array($post2->ID, $options['done'])) {
				$lenDiff = strlen($post->post_content) - strlen($post2->post_content);
				if ($lenDiff > -200 AND $lenDiff < 200) {
					similar_text($post2->post_content, $post->post_content, $p);
					if ($p > $options['similarity']) {
						if ($post->post_date > $post2->post_date) {
							$options['found'][] = array($post->ID, $post2->ID, intval(round($p)));
							$new_duplicates[] = array($post->ID, $post2->ID, intval(round($p)), get_admin_url(), $post->post_title, $post2->post_title);
						} else {
							$options['found'][] = array($post2->ID, $post->ID, intval(round($p)));
							$new_duplicates[] = array($post2->ID, $post->ID, intval(round($p)), get_admin_url(), $post2->post_title, $post->post_title);
						}
					}
				}
			}
		}
		if ($options['post2_offset'] == 0) {
			$options['done'][] = $post->ID;
		}
	}

	update_option('find_duplicates_data', $options);
	return json_encode(array(count($options['done']), $log, $new_duplicates, count($options['found'])));
}


function get_duplicate_results()
{
	echo find_similar_posts($_POST['startnew'], array('statuses' => $_POST['statuses'], 'types' => $_POST['types'], 'similarity' => $_POST['similarity']));
	die;
}

function remove_result()
{
	$data = get_option('find_duplicates_data', null);
	if (wp_delete_post($_POST['id'], false) != false) {
		foreach ($data['found'] as $key => $element) {
			if ($element[0] == $_POST['id']) {
				unset($data['found'][$key]);
			}
		}
		update_option('find_duplicates_data', $data);
		echo json_encode(array(1, $_POST['id']));
	} else {
		echo json_encode(array(0, $_POST['id']));
	}
	die;
}

function get_posts_count()
{
	$statuses = $_POST['statuses'];
	$post_type = $_POST['types'];
	$count = 0;
	if (is_array($statuses)) {
		$post_statuses = "'" . implode("','", $statuses) . "'";
		global $wpdb;
		$posts = $wpdb->get_results("SELECT COUNT(ID) as count FROM $wpdb->posts WHERE post_type='" . $post_type . "' AND post_status IN(" . $post_statuses . ") ");
		$count = $posts[0]->count;
	}
	echo json_encode(array(intval($count)));
	die;
}

add_action('wp_ajax_get_duplicate_results', 'get_duplicate_results');
add_action('wp_ajax_get_posts_count', 'get_posts_count');
add_action('wp_ajax_remove_result', 'remove_result');

?>
